@@ -23,9 +23,17 @@
  * LIABILITY, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE
  * OF THIS SOFTWARE.
  *
- * $Id: oid.h,v 1.21 2003/11/19 13:47:20 mike Exp $
+ * $Id: oid.h,v 1.23 2004/12/22 23:45:13 adam Exp $
  */
 
+/**
+ * \file oid.h
+ * \brief Header for OID database
+ *
+ * More or less protocol-transparent OID database.
+ * We could (and should?) extend this so that the user app can add new
+ * entries to the list at initialization.
+ */
 #ifndef OID_H
 #define OID_H
 
@@ -221,6 +229,7 @@ typedef enum oid_value
 /* 120 */
 
     VAL_MARC21FIN,
+    VAL_CHARNEG4,
 
 /* VAL_DYNAMIC must have highest value */
     VAL_DYNAMIC,
