@@ -1,8 +1,16 @@
-/* 
- $ $Id: charneg.c,v 1.1 2003/10/27 12:21:30 adam Exp $
- * Helper functions for Character Set and Language Negotiation - 3
+/*
+ * Copyright (c) 2002-2004, Index Data
+ * See the file LICENSE for details.
+ *
+ * $Id: charneg.c,v 1.2 2004/10/15 00:19:00 adam Exp $
  */
 
+/** 
+ * \file charneg.c
+ * \brief Implements Z39.50 Charset negotiation utilities
+ *
+ * Helper functions for Character Set and Language Negotiation - 3
+ */
 #include <stdio.h>
 #include <yaz/otherinfo.h>
 #include <yaz/z-charneg.h>
