@@ -1,7 +1,7 @@
-/* A Bison parser, made by GNU Bison 1.875a.  */
+/* A Bison parser, made by GNU Bison 1.875d.  */
 
 /* Skeleton parser for Yacc-like parsing with Bison,
-   Copyright (C) 1984, 1989, 1990, 2000, 2001, 2002, 2003 Free Software Foundation, Inc.
+   Copyright (C) 1984, 1989, 1990, 2000, 2001, 2002, 2003, 2004 Free Software Foundation, Inc.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -87,6 +87,13 @@
 /* Copy the first part of user declarations.  */
 #line 11 "cql.y"
 
+/** 
+ * \file cql.c
+ * \brief Implements CQL parser.
+ *
+ * This is a YACC parser, but since it must be reentrant, Bison is required.
+ * The original source file is cql.y.
+ */
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -94,12 +101,18 @@
 #include <yaz/xmalloc.h>
 #include <yaz/nmem.h>
 #include <yaz/cql.h>
-    
+
+    /** Node in the LALR parse tree. */
     typedef struct {
+	/** Inhereted attribute: relation */
         struct cql_node *rel;
+	/** Synthesized attribute: CQL node */
         struct cql_node *cql;
+	/** string buffer with token */
         char *buf;
+	/** length of token */
         size_t len;
+	/** size of buffer (len <= size) */
         size_t size;
     } token;        
 
@@ -148,22 +161,29 @@ typedef int YYSTYPE;
 
 
 /* Line 214 of yacc.c.  */
-#line 152 "y.tab.c"
+#line 165 "y.tab.c"
 
 #if ! defined (yyoverflow) || YYERROR_VERBOSE
 
+# ifndef YYFREE
+#  define YYFREE free
+# endif
+# ifndef YYMALLOC
+#  define YYMALLOC malloc
+# endif
+
 /* The parser invokes alloca or malloc; define the necessary symbols.  */
 
-# if YYSTACK_USE_ALLOCA
-#  define YYSTACK_ALLOC alloca
+# ifdef YYSTACK_USE_ALLOCA
+#  if YYSTACK_USE_ALLOCA
+#   define YYSTACK_ALLOC alloca
+#  endif
 # else
-#  ifndef YYSTACK_USE_ALLOCA
-#   if defined (alloca) || defined (_ALLOCA_H)
-#    define YYSTACK_ALLOC alloca
-#   else
-#    ifdef __GNUC__
-#     define YYSTACK_ALLOC __builtin_alloca
-#    endif
+#  if defined (alloca) || defined (_ALLOCA_H)
+#   define YYSTACK_ALLOC alloca
+#  else
+#   ifdef __GNUC__
+#    define YYSTACK_ALLOC __builtin_alloca
 #   endif
 #  endif
 # endif
@@ -176,20 +196,20 @@ typedef int YYSTYPE;
 #   include <stdlib.h> /* INFRINGES ON USER NAME SPACE */
 #   define YYSIZE_T size_t
 #  endif
-#  define YYSTACK_ALLOC malloc
-#  define YYSTACK_FREE free
+#  define YYSTACK_ALLOC YYMALLOC
+#  define YYSTACK_FREE YYFREE
 # endif
 #endif /* ! defined (yyoverflow) || YYERROR_VERBOSE */
 
 
 #if (! defined (yyoverflow) \
      && (! defined (__cplusplus) \
-	 || (YYSTYPE_IS_TRIVIAL)))
+	 || (defined (YYSTYPE_IS_TRIVIAL) && YYSTYPE_IS_TRIVIAL)))
 
 /* A type that is properly aligned for any stack member.  */
 union yyalloc
 {
-  short yyss;
+  short int yyss;
   YYSTYPE yyvs;
   };
 
@@ -199,13 +219,13 @@ union yyalloc
 /* The size of an array large to enough to hold all stacks, each with
    N elements.  */
 # define YYSTACK_BYTES(N) \
-     ((N) * (sizeof (short) + sizeof (YYSTYPE))				\
+     ((N) * (sizeof (short int) + sizeof (YYSTYPE))			\
       + YYSTACK_GAP_MAXIMUM)
 
 /* Copy COUNT objects from FROM to TO.  The source and destination do
    not overlap.  */
 # ifndef YYCOPY
-#  if 1 < __GNUC__
+#  if defined (__GNUC__) && 1 < __GNUC__
 #   define YYCOPY(To, From, Count) \
       __builtin_memcpy (To, From, (Count) * sizeof (*(From)))
 #  else
@@ -241,7 +261,7 @@ union yyalloc
 #if defined (__STDC__) || defined (__cplusplus)
    typedef signed char yysigned_char;
 #else
-   typedef short yysigned_char;
+   typedef short int yysigned_char;
 #endif
 
 /* YYFINAL -- State number of the termination state. */
@@ -328,11 +348,11 @@ static const yysigned_char yyrhs[] =
 /* YYRLINE[YYN] -- source line where rule number YYN was defined.  */
 static const unsigned char yyrline[] =
 {
-       0,    53,    53,    53,    62,    63,    70,    72,    72,    86,
-      86,    93,    99,    99,   106,   106,   111,   111,   121,   121,
-     121,   121,   124,   132,   140,   146,   147,   148,   149,   150,
-     151,   155,   156,   157,   158,   159,   160,   161,   165,   168,
-     169,   170,   171,   172
+       0,    66,    66,    66,    76,    77,    84,    86,    86,   101,
+     101,   108,   114,   114,   121,   121,   127,   127,   138,   138,
+     138,   138,   141,   150,   159,   165,   166,   167,   168,   169,
+     170,   174,   175,   176,   177,   178,   179,   180,   184,   187,
+     188,   189,   190,   191
 };
 #endif
 
@@ -341,10 +361,10 @@ static const unsigned char yyrline[] =
    First, the terminals, then, starting at YYNTOKENS, nonterminals. */
 static const char *const yytname[] =
 {
-  "$end", "error", "$undefined", "TERM", "AND", "OR", "NOT", "PROX", "GE", 
-  "LE", "NE", "'('", "')'", "'>'", "'='", "'/'", "'<'", "$accept", "top", 
-  "@1", "cqlQuery1", "cqlQuery", "@2", "searchClause", "@3", "@4", "@5", 
-  "@6", "boolean", "modifiers", "mrelation", "relation", "index", 
+  "$end", "error", "$undefined", "TERM", "AND", "OR", "NOT", "PROX", "GE",
+  "LE", "NE", "'('", "')'", "'>'", "'='", "'/'", "'<'", "$accept", "top",
+  "@1", "cqlQuery1", "cqlQuery", "@2", "searchClause", "@3", "@4", "@5",
+  "@6", "boolean", "modifiers", "mrelation", "relation", "index",
   "searchTerm", 0
 };
 #endif
@@ -352,7 +372,7 @@ static const char *const yytname[] =
 # ifdef YYPRINT
 /* YYTOKNUM[YYLEX-NUM] -- Internal token number corresponding to
    token YYLEX-NUM.  */
-static const unsigned short yytoknum[] =
+static const unsigned short int yytoknum[] =
 {
        0,   256,   257,   258,   259,   260,   261,   262,   263,   264,
      265,    40,    41,    62,    61,    47,    60
@@ -483,7 +503,7 @@ static const unsigned char yystos[] =
 
 #define YYACCEPT	goto yyacceptlab
 #define YYABORT		goto yyabortlab
-#define YYERROR		goto yyerrlab1
+#define YYERROR		goto yyerrorlab
 
 
 /* Like YYERROR except do call yyerror.  This remains here temporarily
@@ -518,11 +538,11 @@ while (0)
    are run).  */
 
 #ifndef YYLLOC_DEFAULT
-# define YYLLOC_DEFAULT(Current, Rhs, N)         \
-  Current.first_line   = Rhs[1].first_line;      \
-  Current.first_column = Rhs[1].first_column;    \
-  Current.last_line    = Rhs[N].last_line;       \
-  Current.last_column  = Rhs[N].last_column;
+# define YYLLOC_DEFAULT(Current, Rhs, N)		\
+   ((Current).first_line   = (Rhs)[1].first_line,	\
+    (Current).first_column = (Rhs)[1].first_column,	\
+    (Current).last_line    = (Rhs)[N].last_line,	\
+    (Current).last_column  = (Rhs)[N].last_column)
 #endif
 
 /* YYLEX -- calling `yylex' with the right arguments.  */
@@ -566,17 +586,17 @@ do {								\
 
 /*------------------------------------------------------------------.
 | yy_stack_print -- Print the state stack from its BOTTOM up to its |
-| TOP (cinluded).                                                   |
+| TOP (included).                                                   |
 `------------------------------------------------------------------*/
 
 #if defined (__STDC__) || defined (__cplusplus)
 static void
-yy_stack_print (short *bottom, short *top)
+yy_stack_print (short int *bottom, short int *top)
 #else
 static void
 yy_stack_print (bottom, top)
-    short *bottom;
-    short *top;
+    short int *bottom;
+    short int *top;
 #endif
 {
   YYFPRINTF (stderr, "Stack now");
@@ -606,9 +626,9 @@ yy_reduce_print (yyrule)
 #endif
 {
   int yyi;
-  unsigned int yylineno = yyrline[yyrule];
+  unsigned int yylno = yyrline[yyrule];
   YYFPRINTF (stderr, "Reducing stack by rule %d (line %u), ",
-             yyrule - 1, yylineno);
+             yyrule - 1, yylno);
   /* Print the symbols being reduced, and their result.  */
   for (yyi = yyprhs[yyrule]; 0 <= yyrhs[yyi]; yyi++)
     YYFPRINTF (stderr, "%s ", yytname [yyrhs[yyi]]);
@@ -645,7 +665,7 @@ int yydebug;
    SIZE_MAX < YYSTACK_BYTES (YYMAXDEPTH)
    evaluated with infinite-precision integer arithmetic.  */
 
-#if YYMAXDEPTH == 0
+#if defined (YYMAXDEPTH) && YYMAXDEPTH == 0
 # undef YYMAXDEPTH
 #endif
 
@@ -843,9 +863,9 @@ int yynerrs;
      to reallocate them elsewhere.  */
 
   /* The state stack.  */
-  short	yyssa[YYINITDEPTH];
-  short *yyss = yyssa;
-  register short *yyssp;
+  short int yyssa[YYINITDEPTH];
+  short int *yyss = yyssa;
+  register short int *yyssp;
 
   /* The semantic value stack.  */
   YYSTYPE yyvsa[YYINITDEPTH];
@@ -882,6 +902,7 @@ int yynerrs;
   yyssp = yyss;
   yyvsp = yyvs;
 
+
   goto yysetstate;
 
 /*------------------------------------------------------------.
@@ -907,7 +928,7 @@ int yynerrs;
 	   these so that the &'s don't force the real ones into
 	   memory.  */
 	YYSTYPE *yyvs1 = yyvs;
-	short *yyss1 = yyss;
+	short int *yyss1 = yyss;
 
 
 	/* Each stack pointer address is followed by the size of the
@@ -935,7 +956,7 @@ int yynerrs;
 	yystacksize = YYMAXDEPTH;
 
       {
-	short *yyss1 = yyss;
+	short int *yyss1 = yyss;
 	union yyalloc *yyptr =
 	  (union yyalloc *) YYSTACK_ALLOC (YYSTACK_BYTES (yystacksize));
 	if (! yyptr)
@@ -1068,15 +1089,16 @@ yyreduce:
   switch (yyn)
     {
         case 2:
-#line 53 "cql.y"
+#line 66 "cql.y"
     { 
-    yyval.rel = cql_node_mk_sc("cql.serverChoice", "scr", 0);
+    yyval.rel = cql_node_mk_sc(((CQL_parser) parm)->nmem,
+			    "cql.serverChoice", "scr", 0);
     ((CQL_parser) parm)->top = 0;
 }
     break;
 
   case 3:
-#line 56 "cql.y"
+#line 70 "cql.y"
     {
     cql_node_destroy(yyval.rel);
     ((CQL_parser) parm)->top = yyvsp[0].cql; 
@@ -1084,7 +1106,7 @@ yyreduce:
     break;
 
   case 5:
-#line 63 "cql.y"
+#line 77 "cql.y"
     {
     cql_node_destroy(yyvsp[-1].cql);
     yyval.cql = 0;
@@ -1092,16 +1114,17 @@ yyreduce:
     break;
 
   case 7:
-#line 72 "cql.y"
+#line 86 "cql.y"
     { 
       yyval.rel = yyvsp[-3].rel;
   }
     break;
 
   case 8:
-#line 74 "cql.y"
+#line 88 "cql.y"
     {
-      struct cql_node *cn = cql_node_mk_boolean(yyvsp[-3].buf);
+      struct cql_node *cn = cql_node_mk_boolean(((CQL_parser) parm)->nmem,
+						yyvsp[-3].buf);
       
       cn->u.boolean.modifiers = yyvsp[-2].cql;
       cn->u.boolean.left = yyvsp[-4].cql;
@@ -1112,7 +1135,7 @@ yyreduce:
     break;
 
   case 9:
-#line 86 "cql.y"
+#line 101 "cql.y"
     { 
       yyval.rel = yyvsp[-1].rel;
       
@@ -1120,31 +1143,31 @@ yyreduce:
     break;
 
   case 10:
-#line 89 "cql.y"
+#line 104 "cql.y"
     {
       yyval.cql = yyvsp[-1].cql;
   }
     break;
 
   case 11:
-#line 93 "cql.y"
+#line 108 "cql.y"
     {
-      struct cql_node *st = cql_node_dup (yyvsp[-1].rel);
-      st->u.st.term = xstrdup(yyvsp[0].buf);
+      struct cql_node *st = cql_node_dup (((CQL_parser) parm)->nmem, yyvsp[-1].rel);
+      st->u.st.term = nmem_strdup(((CQL_parser)parm)->nmem, yyvsp[0].buf);
       yyval.cql = st;
   }
     break;
 
   case 12:
-#line 99 "cql.y"
+#line 114 "cql.y"
     {
-      yyval.rel = cql_node_mk_sc(yyvsp[-2].buf, yyvsp[-1].buf, 0);
+      yyval.rel = cql_node_mk_sc(((CQL_parser) parm)->nmem, yyvsp[-2].buf, yyvsp[-1].buf, 0);
       yyval.rel->u.st.modifiers = yyvsp[0].cql;
   }
     break;
 
   case 13:
-#line 102 "cql.y"
+#line 117 "cql.y"
     {
       yyval.cql = yyvsp[0].cql;
       cql_node_destroy(yyvsp[-1].rel);
@@ -1152,37 +1175,40 @@ yyreduce:
     break;
 
   case 14:
-#line 106 "cql.y"
+#line 121 "cql.y"
     {
       yyval.rel = yyvsp[-4].rel;
   }
     break;
 
   case 15:
-#line 108 "cql.y"
+#line 123 "cql.y"
     {
-    yyval.cql = cql_apply_prefix(yyvsp[0].cql, yyvsp[-4].buf, yyvsp[-2].buf);
+    yyval.cql = cql_apply_prefix(((CQL_parser) parm)->nmem,
+			      yyvsp[0].cql, yyvsp[-4].buf, yyvsp[-2].buf);
   }
     break;
 
   case 16:
-#line 111 "cql.y"
+#line 127 "cql.y"
     {
       yyval.rel = yyvsp[-2].rel;
   }
     break;
 
   case 17:
-#line 113 "cql.y"
+#line 129 "cql.y"
     {
-    yyval.cql = cql_apply_prefix(yyvsp[0].cql, 0, yyvsp[-2].buf);
+    yyval.cql = cql_apply_prefix(((CQL_parser) parm)->nmem, 
+			      yyvsp[0].cql, 0, yyvsp[-2].buf);
    }
     break;
 
   case 22:
-#line 125 "cql.y"
+#line 142 "cql.y"
     { 
-    struct cql_node *mod = cql_node_mk_sc(yyvsp[0].buf, "=", 0);
+    struct cql_node *mod = cql_node_mk_sc(((CQL_parser)parm)->nmem,
+					  yyvsp[0].buf, "=", 0);
 
     mod->u.st.modifiers = yyvsp[-2].cql;
     yyval.cql = mod;
@@ -1190,9 +1216,10 @@ yyreduce:
     break;
 
   case 23:
-#line 133 "cql.y"
+#line 151 "cql.y"
     {
-    struct cql_node *mod = cql_node_mk_sc(yyvsp[-2].buf, yyvsp[-1].buf, yyvsp[0].buf);
+    struct cql_node *mod = cql_node_mk_sc(((CQL_parser)parm)->nmem,
+					  yyvsp[-2].buf, yyvsp[-1].buf, yyvsp[0].buf);
 
     mod->u.st.modifiers = yyvsp[-4].cql;
     yyval.cql = mod;
@@ -1200,7 +1227,7 @@ yyreduce:
     break;
 
   case 24:
-#line 140 "cql.y"
+#line 159 "cql.y"
     { 
     yyval.cql = 0;
 }
@@ -1209,8 +1236,8 @@ yyreduce:
 
     }
 
-/* Line 999 of yacc.c.  */
-#line 1214 "y.tab.c"
+/* Line 1010 of yacc.c.  */
+#line 1241 "y.tab.c"
 
   yyvsp -= yylen;
   yyssp -= yylen;
@@ -1251,18 +1278,33 @@ yyerrlab:
 	{
 	  YYSIZE_T yysize = 0;
 	  int yytype = YYTRANSLATE (yychar);
+	  const char* yyprefix;
 	  char *yymsg;
-	  int yyx, yycount;
+	  int yyx;
 
-	  yycount = 0;
 	  /* Start YYX at -YYN if negative to avoid negative indexes in
 	     YYCHECK.  */
-	  for (yyx = yyn < 0 ? -yyn : 0;
-	       yyx < (int) (sizeof (yytname) / sizeof (char *)); yyx++)
+	  int yyxbegin = yyn < 0 ? -yyn : 0;
+
+	  /* Stay within bounds of both yycheck and yytname.  */
+	  int yychecklim = YYLAST - yyn;
+	  int yyxend = yychecklim < YYNTOKENS ? yychecklim : YYNTOKENS;
+	  int yycount = 0;
+
+	  yyprefix = ", expecting ";
+	  for (yyx = yyxbegin; yyx < yyxend; ++yyx)
 	    if (yycheck[yyx + yyn] == yyx && yyx != YYTERROR)
-	      yysize += yystrlen (yytname[yyx]) + 15, yycount++;
-	  yysize += yystrlen ("syntax error, unexpected ") + 1;
-	  yysize += yystrlen (yytname[yytype]);
+	      {
+		yysize += yystrlen (yyprefix) + yystrlen (yytname [yyx]);
+		yycount += 1;
+		if (yycount == 5)
+		  {
+		    yysize = 0;
+		    break;
+		  }
+	      }
+	  yysize += (sizeof ("syntax error, unexpected ")
+		     + yystrlen (yytname[yytype]));
 	  yymsg = (char *) YYSTACK_ALLOC (yysize);
 	  if (yymsg != 0)
 	    {
@@ -1271,16 +1313,13 @@ yyerrlab:
 
 	      if (yycount < 5)
 		{
-		  yycount = 0;
-		  for (yyx = yyn < 0 ? -yyn : 0;
-		       yyx < (int) (sizeof (yytname) / sizeof (char *));
-		       yyx++)
+		  yyprefix = ", expecting ";
+		  for (yyx = yyxbegin; yyx < yyxend; ++yyx)
 		    if (yycheck[yyx + yyn] == yyx && yyx != YYTERROR)
 		      {
-			const char *yyq = ! yycount ? ", expecting " : " or ";
-			yyp = yystpcpy (yyp, yyq);
+			yyp = yystpcpy (yyp, yyprefix);
 			yyp = yystpcpy (yyp, yytname[yyx]);
-			yycount++;
+			yyprefix = " or ";
 		      }
 		}
 	      yyerror (yymsg);
@@ -1301,25 +1340,27 @@ yyerrlab:
       /* If just tried and failed to reuse lookahead token after an
 	 error, discard it.  */
 
-      /* Return failure if at end of input.  */
-      if (yychar == YYEOF)
+      if (yychar <= YYEOF)
         {
-	  /* Pop the error token.  */
-          YYPOPSTACK;
-	  /* Pop the rest of the stack.  */
-	  while (yyss < yyssp)
-	    {
-	      YYDSYMPRINTF ("Error: popping", yystos[*yyssp], yyvsp, yylsp);
-	      yydestruct (yystos[*yyssp], yyvsp);
-	      YYPOPSTACK;
-	    }
-	  YYABORT;
+          /* If at end of input, pop the error token,
+	     then the rest of the stack, then return failure.  */
+	  if (yychar == YYEOF)
+	     for (;;)
+	       {
+		 YYPOPSTACK;
+		 if (yyssp == yyss)
+		   YYABORT;
+		 YYDSYMPRINTF ("Error: popping", yystos[*yyssp], yyvsp, yylsp);
+		 yydestruct (yystos[*yyssp], yyvsp);
+	       }
         }
+      else
+	{
+	  YYDSYMPRINTF ("Error: discarding", yytoken, &yylval, &yylloc);
+	  yydestruct (yytoken, &yylval);
+	  yychar = YYEMPTY;
 
-      YYDSYMPRINTF ("Error: discarding", yytoken, &yylval, &yylloc);
-      yydestruct (yytoken, &yylval);
-      yychar = YYEMPTY;
-
+	}
     }
 
   /* Else will try to reuse lookahead token after shifting the error
@@ -1327,9 +1368,27 @@ yyerrlab:
   goto yyerrlab1;
 
 
-/*----------------------------------------------------.
-| yyerrlab1 -- error raised explicitly by an action.  |
-`----------------------------------------------------*/
+/*---------------------------------------------------.
+| yyerrorlab -- error raised explicitly by YYERROR.  |
+`---------------------------------------------------*/
+yyerrorlab:
+
+#ifdef __GNUC__
+  /* Pacify GCC when the user code never invokes YYERROR and the label
+     yyerrorlab therefore never appears in user code.  */
+  if (0)
+     goto yyerrorlab;
+#endif
+
+  yyvsp -= yylen;
+  yyssp -= yylen;
+  yystate = *yyssp;
+  goto yyerrlab1;
+
+
+/*-------------------------------------------------------------.
+| yyerrlab1 -- common code for both syntax error and YYERROR.  |
+`-------------------------------------------------------------*/
 yyerrlab1:
   yyerrstatus = 3;	/* Each real token shifted decrements this.  */
 
@@ -1353,9 +1412,8 @@ yyerrlab1:
 
       YYDSYMPRINTF ("Error: popping", yystos[*yyssp], yyvsp, yylsp);
       yydestruct (yystos[yystate], yyvsp);
-      yyvsp--;
-      yystate = *--yyssp;
-
+      YYPOPSTACK;
+      yystate = *yyssp;
       YY_STACK_PRINT (yyss, yyssp);
     }
 
@@ -1404,7 +1462,7 @@ yyreturn:
 }
 
 
-#line 175 "cql.y"
+#line 194 "cql.y"
 
 
 int yyerror(char *s)
@@ -1412,15 +1470,18 @@ int yyerror(char *s)
     return 0;
 }
 
-/*
- * bison lexer for CQL.
+/**
+ * putb is a utility that puts one character to the string
+ * in current lexical token. This routine deallocates as
+ * necessary using NMEM.
  */
 
 static void putb(YYSTYPE *lval, CQL_parser cp, int c)
 {
     if (lval->len+1 >= lval->size)
     {
-        char *nb = nmem_malloc(cp->nmem, (lval->size = lval->len * 2 + 20));
+        char *nb = (char *)
+	    nmem_malloc(cp->nmem, (lval->size = lval->len * 2 + 20));
         memcpy (nb, lval->buf, lval->len);
         lval->buf = nb;
     }
@@ -1430,6 +1491,10 @@ static void putb(YYSTYPE *lval, CQL_parser cp, int c)
 }
 
 
+/**
+ * yylex returns next token for Bison to be read. In this
+ * case one of the CQL terminals are returned.
+ */
 int yylex(YYSTYPE *lval, void *vp)
 {
     CQL_parser cp = (CQL_parser) vp;
@@ -1438,7 +1503,7 @@ int yylex(YYSTYPE *lval, void *vp)
     lval->rel = 0;
     lval->len = 0;
     lval->size = 10;
-    lval->buf = nmem_malloc(cp->nmem, lval->size);
+    lval->buf = (char *) nmem_malloc(cp->nmem, lval->size);
     lval->buf[0] = '\0';
     do
     {
