@@ -1,10 +1,15 @@
 /*
- * Copyright (c) 1995-2003, Index Data.
+ * Copyright (c) 1995-2004, Index Data.
  * See the file LICENSE for details.
- * Sebastian Hammer, Adam Dickmeiss
  *
- * $Id: matchstr.c,v 1.1 2003/10/27 12:21:31 adam Exp $
+ * $Id: matchstr.c,v 1.3 2004/12/30 00:14:00 adam Exp $
  */
+
+/**
+ * \file matchstr.c
+ * \brief Implements loose string matching 
+ */
+
 #if HAVE_CONFIG_H
 #include <config.h>
 #endif
@@ -25,8 +30,8 @@ int yaz_matchstr(const char *s1, const char *s2)
 {
     while (*s1 && *s2)
     {
-	char c1 = *s1;
-	char c2 = *s2;
+	unsigned char c1 = *s1;
+	unsigned char c2 = *s2;
 
         if (c2 == '?')
             return 0;
