@@ -1,14 +1,19 @@
 /*
- * Copyright (c) 1995-2003, Index Data.
+ * Copyright (c) 1995-2004, Index Data.
  * See the file LICENSE for details.
- * Sebastian Hammer, Adam Dickmeiss
  *
- * $Id: nmem.c,v 1.3 2004/01/15 10:16:27 adam Exp $
+ * $Id: nmem.c,v 1.9 2005/01/04 01:25:20 adam Exp $
  */
 
-/*
+/**
+ * \file nmem.c
+ * \brief Implements Nibble Memory
+ *
  * This is a simple and fairly wasteful little module for nibble memory
  * allocation. Evemtually we'll put in something better.
+ *
+ * FIXME - it also has some semaphore stuff, and stuff to handle errno.
+ *         These should be moved to some other place!
  */
 #if HAVE_CONFIG_H
 #include <config.h>
@@ -35,6 +40,10 @@
 #endif
 
 #define NMEM_CHUNK (4*1024)
+#define NMEM_ALIGN (sizeof(long)*2)
+
+static int log_level=0;
+static int log_level_initialized=0;
 
 #ifdef WIN32
 static CRITICAL_SECTION critical_section;
@@ -67,6 +76,12 @@ struct nmem_mutex {
 
 YAZ_EXPORT void nmem_mutex_create(NMEM_MUTEX *p)
 {
+    if (!log_level_initialized)
+    {
+        log_level=yaz_log_module_level("nmem");
+        log_level_initialized=1;
+    }
+
     NMEM_ENTER;
     if (!*p)
     {
@@ -141,15 +156,15 @@ static void free_block(nmem_block *p)
     memset(p->buf, 'Y', p->size);
     p->next = freelist;
     freelist = p;
-#if NMEM_DEBUG
-    yaz_log (LOG_DEBUG, "nmem free_block p=%p", p);
-#endif
+    if (log_level)
+        yaz_log (log_level, "nmem free_block p=%p", p);
 }
 
 #if NMEM_DEBUG
 void nmem_print_list (void)
 {
-    nmem_print_list_l(LOG_DEBUG);
+    if(log_level)
+        nmem_print_list_l(log_level);
 }
 
 void nmem_print_list_l (int level)
@@ -171,17 +186,16 @@ static nmem_block *get_block(int size)
 {
     nmem_block *r, *l;
 
-#if NMEM_DEBUG
-    yaz_log (LOG_DEBUG, "nmem get_block size=%d", size);
-#endif
+    if (log_level)
+        yaz_log (log_level, "nmem get_block size=%d", size);
+
     for (r = freelist, l = 0; r; l = r, r = r->next)
     	if (r->size >= size)
 	    break;
     if (r)
     {
-#if NMEM_DEBUG
-	yaz_log (LOG_DEBUG, "nmem get_block found free block p=%p", r);
-#endif
+        if (log_level)
+	    yaz_log (log_level, "nmem get_block found free block p=%p", r);
     	if (l)
 	    l->next = r->next;
 	else
@@ -193,9 +207,9 @@ static nmem_block *get_block(int size)
 
 	if (get < size)
 	    get = size;
-#if NMEM_DEBUG
-	yaz_log (LOG_DEBUG, "nmem get_block alloc new block size=%d", get);
-#endif
+        if(log_level)
+	    yaz_log (log_level, "nmem get_block alloc new block size=%d", get);
+
 	r = (nmem_block *)xmalloc(sizeof(*r));
 	r->buf = (char *)xmalloc(r->size = get);
     }
@@ -206,10 +220,8 @@ static nmem_block *get_block(int size)
 void nmem_reset(NMEM n)
 {
     nmem_block *t;
-
-#if NMEM_DEBUG
-    yaz_log (LOG_DEBUG, "nmem_reset p=%p", n);
-#endif
+    
+    yaz_log (log_level, "nmem_reset p=%p", n);
     if (!n)
 	return;
     NMEM_ENTER;
@@ -233,12 +245,13 @@ void *nmem_malloc(NMEM n, int size)
     char *r;
 
 #if NMEM_DEBUG
-    yaz_log (LOG_DEBUG, "%s:%d: nmem_malloc p=%p size=%d", file, line,
-                     n, size);
+    if (log_level)
+        yaz_log (log_level, "%s:%d: nmem_malloc p=%p size=%d", 
+                file, line, n, size);
 #endif
     if (!n)
     {
-        yaz_log (LOG_FATAL, "calling nmem_malloc with an null pointer");
+        yaz_log (YLOG_FATAL, "calling nmem_malloc with an null pointer");
         abort ();
     }
 #ifdef WIN32
@@ -254,7 +267,7 @@ void *nmem_malloc(NMEM n, int size)
     }
     r = p->buf + p->top;
     /* align size */
-    p->top += (size + (sizeof(long) - 1)) & ~(sizeof(long) - 1);
+    p->top += (size + (NMEM_ALIGN - 1)) & ~(NMEM_ALIGN - 1);
     n->total += size;
     NMEM_LEAVE;
     return r;
@@ -275,6 +288,11 @@ NMEM nmem_create(void)
 #if NMEM_DEBUG
     struct nmem_debug_info *debug_p;
 #endif
+    if (!log_level_initialized)
+    {
+        log_level=yaz_log_module_level("nmem");
+        log_level_initialized=1;
+    }
     
     NMEM_ENTER;
     nmem_active_no++;
@@ -286,7 +304,7 @@ NMEM nmem_create(void)
     NMEM_LEAVE;
 
 #if NMEM_DEBUG
-    yaz_log (LOG_DEBUG, "%s:%d: nmem_create %d p=%p", file, line,
+    yaz_log (YLOG_DEBUG, "%s:%d: nmem_create %d p=%p", file, line,
                      nmem_active_no, r);
 #endif
     r->blocks = 0;
@@ -297,7 +315,7 @@ NMEM nmem_create(void)
     for (debug_p = nmem_debug_list; debug_p; debug_p = debug_p->next)
 	if (debug_p->p == r)
 	{
-	    yaz_log (LOG_FATAL, "multi used block in nmem");
+	    yaz_log (YLOG_FATAL, "multi used block in nmem");
 	    abort ();
 	}
     debug_p = xmalloc (sizeof(*debug_p));
@@ -327,7 +345,7 @@ void nmem_destroy(NMEM n)
 	return;
     
 #if NMEM_DEBUG
-    yaz_log (LOG_DEBUG, "%s:%d: nmem_destroy %d p=%p", file, line,
+    yaz_log (log_level, "%s:%d: nmem_destroy %d p=%p", file, line,
                      nmem_active_no-1, n);
     NMEM_ENTER;
     for (debug_p = &nmem_debug_list; *debug_p; debug_p = &(*debug_p)->next)
@@ -343,7 +361,7 @@ void nmem_destroy(NMEM n)
     nmem_print_list();
     if (!ok)
     {
-	yaz_log (LOG_WARN, "%s:%d destroying unallocated nmem block p=%p",
+	yaz_log (YLOG_WARN, "%s:%d destroying unallocated nmem block p=%p",
 		 file, line, n);
 	return;
     }
@@ -359,7 +377,7 @@ void nmem_destroy(NMEM n)
 void nmem_transfer (NMEM dst, NMEM src)
 {
     nmem_block *t;
-    while ((t=src->blocks))
+    while ((t = src->blocks))
     {
 	src->blocks = t->next;
 	t->next = dst->blocks;
@@ -381,12 +399,18 @@ void nmem_critical_leave (void)
 
 void nmem_init (void)
 {
+    if (!log_level_initialized)
+    {
+        log_level=yaz_log_module_level("nmem");
+        log_level_initialized=1;
+    }
+    
     if (++nmem_init_flag == 1)
     {
 #ifdef WIN32
 	InitializeCriticalSection(&critical_section);
 #elif YAZ_GNU_THREADS
-	yaz_log (LOG_LOG, "pth_init");
+	yaz_log (log_level, "pth_init"); /* ??? */
         pth_init ();
 #endif
 	nmem_active_no = 0;
@@ -449,9 +473,18 @@ void yaz_set_errno(int v)
 
 void yaz_strerror(char *buf, int max)
 {
-    char *cp;
 #ifdef WIN32
-    DWORD err = GetLastError();
+    DWORD err;
+#endif
+    char *cp;
+    if (!log_level_initialized)
+    {
+        log_level = yaz_log_module_level("nmem");
+        log_level_initialized = 1;
+    }
+    
+#ifdef WIN32
+    err = GetLastError();
     if (err)
     {
         FormatMessage(
@@ -482,8 +515,8 @@ void yaz_strerror(char *buf, int max)
 #endif
 /* UNIX */
 #endif
-    if ((cp=strrchr(buf, '\n')))
+    if ((cp = strrchr(buf, '\n')))
 	*cp = '\0';
-    if ((cp=strrchr(buf, '\r')))
+    if ((cp = strrchr(buf, '\r')))
 	*cp = '\0';
 }
