@@ -1,9 +1,13 @@
 /*
- * Copyright (c) 1995-2003, Index Data
+ * Copyright (c) 1995-2004, Index Data
  * See the file LICENSE for details.
- * Sebastian Hammer, Adam Dickmeiss
  *
- * $Id: odr_bool.c,v 1.1 2003/10/27 12:21:33 adam Exp $
+ * $Id: odr_bool.c,v 1.3 2004/10/15 00:19:00 adam Exp $
+ */
+
+/**
+ * \file odr_bool.c
+ * \brief Implements ODR BOOLEAN codec
  */
 #if HAVE_CONFIG_H
 #include <config.h>
@@ -34,7 +38,7 @@ int odr_bool(ODR o, int **p, int opt, const char *name)
     if (o->direction == ODR_PRINT)
     {
 	odr_prname(o, name);
-    	fprintf(o->print, "%s\n", (**p ? "TRUE" : "FALSE"));
+    	odr_printf(o, "%s\n", (**p ? "TRUE" : "FALSE"));
     	return 1;
     }
     if (cons)
