@@ -1,10 +1,15 @@
 /*
- * Copyright (c) 1995-2003, Index Data
+ * Copyright (c) 1995-2004, Index Data
  * See the file LICENSE for details.
- * Sebastian Hammer, Adam Dickmeiss
  *
- * $Id: odr_choice.c,v 1.1 2003/10/27 12:21:33 adam Exp $
+ * $Id: odr_choice.c,v 1.3 2004/10/15 00:19:00 adam Exp $
  */
+
+/**
+ * \file odr_choice.c
+ * \brief Implements ODR CHOICE codec
+ */
+
 #if HAVE_CONFIG_H
 #include <config.h>
 #endif
@@ -33,7 +38,7 @@ int odr_choice(ODR o, Odr_arm arm[], void *p, void *whichp,
 	if (name)
 	{
 	    odr_prname(o, name);
-	    fprintf (o->print, "choice\n");
+	    odr_printf(o, "choice\n");
 	}
     }
     for (i = 0; arm[i].fun; i++)
