@@ -2,7 +2,11 @@
  * Copyright (c) 1995-2004, Index Data
  * See the file LICENSE for details.
  *
- * $Id: odr_util.c,v 1.3 2004/01/05 14:46:52 adam Exp $
+ * $Id: odr_util.c,v 1.6 2004/12/30 00:14:38 adam Exp $
+ */
+/**
+ * \file odr_util.c
+ * \brief Implements various ODR utilities
  */
 #if HAVE_CONFIG_H
 #include <config.h>
@@ -17,9 +21,9 @@
 void odr_prname(ODR o, const char *name)
 {
     if (name)
-	fprintf (o->print, "%*s%s ", o->indent*4, "", name);
+	odr_printf(o, "%*s%s ", o->indent*4, "", name);
     else
-	fprintf (o->print, "%*s", o->indent*4, "");
+	odr_printf(o, "%*s", o->indent*4, "");
 }
 
 int odp_more_chunks(ODR o, const unsigned char *base, int len)
@@ -65,7 +69,7 @@ Odr_oid *odr_getoidbystr_nmem(NMEM nmem, const char *str)
     const char *p = str;
     Odr_oid *ret;
 
-    if (!isdigit(*str))
+    if (!isdigit(*(const unsigned char *) str))
 	return 0;
     while ((p = strchr(p, '.')))
 	num++, p++;
